; ***************************************************************************************
; ***                                                                                 ***
; *** This code requires a modified DASM which was posted to [stella] at 02/19/2001 ! ***
; ***                                                                                 ***
; ***************************************************************************************

    processor 6502
    include vcs.h


;===============================================================================
; T I A - C O N S T A N T S
;===============================================================================

BM_SIZE_1   = %000000
BM_SIZE_2   = %010000
BM_SIZE_4   = %100000
BM_SIZE_8   = %110000

HMOVE_L7    = $70
HMOVE_L6    = $60
HMOVE_L5    = $50
HMOVE_L4    = $40
HMOVE_L3    = $30
HMOVE_L2    = $20
HMOVE_L1    = $10
HMOVE_0     = $00
HMOVE_R1    = $f0
HMOVE_R2    = $e0
HMOVE_R3    = $d0
HMOVE_R4    = $c0
HMOVE_R5    = $b0
HMOVE_R6    = $a0
HMOVE_R7    = $90
HMOVE_R8    = $80

DISABLE_BM  = %00
ENABLE_BM   = %10


;===============================================================================
; U S E R - C O N S T A N T S
;===============================================================================

TOMBSTONEHEIGHT = 8  ; 8*2 = 16 (single scanlines) high per tombstone
TOMBSTONEROWS   = 11 ; 11 rows of tombstones per screen

CARHEIGHT       = 8  ; 8 doubled scanlines (used as offset)
ZOMBIE_H        = 8


;===============================================================================
; Z P - V A R I A B L E S
;===============================================================================

    SEG.U   variables
    ORG     $80

frameCnt    ds 1
tmpVar      ds 1

krnlNum     ds 1
vecKernel   ds 2

P0_Y        ds 1
P1_Y        ds 1
M0_Y        ds 1
M1_Y        ds 1

P0_X        ds 1
P1_X        ds 1
M0_X        ds 1
M1_X        ds 1

P0_YFullres ds 1
P1_YFullres ds 1
M0_YFullres ds 1
M1_YFullres ds 1

P0_Ptr      ds 2
P1_Ptr      ds 2
M0_Ptr      ds 2
M1_Ptr      ds 2


;===============================================================================
; M A C R O S
;===============================================================================

  MAC SLEEP
    IF {1} = 1
      ECHO "ERROR: SLEEP 1 not allowed !"
      END
    ENDIF
    IF {1} & 1
      nop $00
      REPEAT ({1}-3)/2
        nop
      REPEND
    ELSE
      REPEAT ({1})/2
        nop
      REPEND
    ENDIF
  ENDM


;===============================================================================
; R O M - C O D E
;===============================================================================

    SEG     Bank0
    ORG     $f000

Start SUBROUTINE
    sei                         ;           Disable interrupts, if there are any.
    cld                         ;           Clear BCD math bit.
    ldx     #0
    txs
    pha                         ;           Set stack to beginning.
    txa
.clearLoop:
    pha
    dex
    bne     .clearLoop

    jsr     GameInit

.mainLoop:
    jsr     VerticalBlank
    jsr     GameCalc
    jsr     DrawScreen
    jsr     OverScan
    jmp     .mainLoop


GameInit SUBROUTINE
    lda     #1
    sta     CTRLPF
    lda     #$02
    sta     COLUPF
    lda     #$36
    sta     COLUP0
    lda     #$c6
    sta     COLUP1

    lda     #50
    sta     P0_YFullres
    lda     #50
    sta     P1_YFullres

    lda     #40
    sta     P0_X
    lda     #48
    sta     P1_X

    lda     #100
    sta     M0_YFullres
    lda     #100
    sta     M1_YFullres

    lda     #80
    sta     M0_X
    lda     #88
    sta     M1_X


    lda     #>CarRotation0
    sta     P0_Ptr+1
    sta     P1_Ptr+1

    lda     #>Ped1_1
    sta     M0_Ptr+1
    sta     M1_Ptr+1

    rts


VerticalBlank SUBROUTINE
    lda     #2
    sta     WSYNC
    sta     VSYNC
    sta     WSYNC
    sta     WSYNC
    lsr
    ldx     #44
    sta     WSYNC
    sta     VSYNC
    stx     TIM64T
    rts


GameCalc SUBROUTINE
    inc     frameCnt

; move the cars:
    lda     frameCnt
    and     #$03
    bne     .skipMoveP0
    ldx     P0_YFullres
    bne     .okP0
    ldx     #[TOMBSTONEROWS*TOMBSTONEHEIGHT+CARHEIGHT]*2
.okP0:
    dex
    stx     P0_YFullres
.skipMoveP0:

    lda     frameCnt
    and     #$07
    bne     .skipMoveP1
    ldx     P1_YFullres
    bne     .okP1
    ldx     #[TOMBSTONEROWS*TOMBSTONEHEIGHT+CARHEIGHT]*2
.okP1:
    dex
    stx     P1_YFullres
.skipMoveP1:

; move the zombies:
    lda     frameCnt
    and     #$03
    bne     .skipMoveM0
    ldx     M0_X
    bne     .okM0_X
    ldx     #159
.okM0_X:
    dex
    stx     M0_X

    ldx     M0_YFullres
    bne     .okM0_Y
    ldx     #[TOMBSTONEROWS*TOMBSTONEHEIGHT+ZOMBIE_H]*2
.okM0_Y:
    dex
    stx     M0_YFullres
.skipMoveM0
;   ...

; calculate kernel variables:
    lda     P0_YFullres
    sta     VDELP0
    lsr
    sta     P0_Y
    lda     #<CarRotation0
    sec
    sbc     #TOMBSTONEROWS*TOMBSTONEHEIGHT
    clc
    adc     P0_Y
    sta     P0_Ptr

    lda     P1_YFullres
    eor     #1
    sta     VDELP1
    eor     #1
    lsr
    sbc     #0
    sta     P1_Y
    lda     #<CarRotation8
    sec
    sbc     #TOMBSTONEROWS*TOMBSTONEHEIGHT
    clc
    adc     P1_Y
    sta     P1_Ptr


    lda     M0_YFullres
    lsr
    sta     M0_Y
    lda     #<Ped1_1
    sec
    sbc     #TOMBSTONEROWS*TOMBSTONEHEIGHT
    clc
    adc     M0_Y
    sta     M0_Ptr

    lda     #TOMBSTONEROWS*TOMBSTONEHEIGHT
    sec
    sbc     M0_Y
    clc
    adc     #ZOMBIE_H-1
    tay
    lda     (M0_Ptr),y
    asl
    asl
    sta     NUSIZ0

    lda     M1_YFullres
    lsr
    sta     M1_Y
    lda     #<Ped1_4
    sec
    sbc     #TOMBSTONEROWS*TOMBSTONEHEIGHT
    clc
    adc     M1_Y
    sta     M1_Ptr

    lda     #TOMBSTONEROWS*TOMBSTONEHEIGHT
    sec
    sbc     M1_Y
    clc
    adc     #ZOMBIE_H-1
    tay
    lda     (M1_Ptr),y
    asl
    asl
    sta     NUSIZ1

; position all objects
    ldx     #3
.loopPos:
    lda     P0_X,x
    jsr     SetPosX
    dex
    bpl     .loopPos

    sta     WSYNC

    sta     HMOVE

; determine which kernel (0..3) to use
    ldy     #0
    lda     M0_YFullres
    lsr
    bcc     .evenM0
    iny
.evenM0:
    lda     M1_YFullres
    lsr
    bcc     .evenM1
    iny
    iny
.evenM1:
    lda     KernelLoTbl,y
    sta     vecKernel
    lda     KernelHiTbl,y
    sta     vecKernel+1

    sta     WSYNC

    sta     HMCLR
    rts

DrawScreen SUBROUTINE
    ldx     #228
.waitTim:
    lda     INTIM
    bne     .waitTim
    sta     WSYNC
    sta     VBLANK
    stx     TIM64T
;---------------------------------------------------------------
    sta     WSYNC
;---------------------------------------

    SLEEP   28              ;28
    ldx     #TOMBSTONEROWS-1; 2
    ldy     #TOMBSTONEROWS*TOMBSTONEHEIGHT-1; 2

    lda     #$c0            ; 2                 enable left and right borders
    sta     PF0             ; 3
    jmp     (vecKernel)     ; 5 = 42
;***************************************

    align   256

Kernel00 SUBROUTINE
.enableM0:                  ;10
    lda     (M0_Ptr),y      ; 5                 bytes stored as: mmmmss00
    sta     HMM0            ; 3
    lda     #ENABLE_BM      ; 2
    sta     ENAM0           ; 3                 TODO: must happen min. 3 cycles later
    bne     .continueM0     ; 3

.disableM0:                 ;12
    nop                     ; 2                 waste six cycles for constant timing
    nop                     ; 2
    nop                     ; 2
    lda     #DISABLE_BM     ; 2
    sta     ENAM0           ; 3                 TODO: must happen min. 3 cycles later
    beq     .continueM0     ; 3
;***************************************

EnterKernel00:              ;           @42
; cycle count:
; - loop overhead           13
; - playfield       28+8 =  36
; - players         18*2 =  36
; - missiles        26*2 =  52
; - HMOVE            3*2 =   6
; - HMCLR            3*2 =   6
; - free               3 =   3
;                          ---
; Total             76*2 = 152 :-)

.contBlock:
    nop                     ; 2
    nop                     ; 2
.nextBlock:                 ;   = 13

    SLEEP   3               ; 3 =  3

; draw zombie #0:
    lda     #ZOMBIE_H-1     ; 2
    dcp     M0_Y            ; 5
    beq     .enableM0       ; 2
    bcc     .disableM0      ; 2
    lda     (M0_Ptr),y      ; 5                 bytes stored as: mmmmss00
    sta     HMM0            ; 3
    asl                     ; 2
    asl                     ; 2
    sta     NUSIZ0          ; 3         @75
.continueM0:                ;   = 26            timing is constant!

;    sta     WSYNC
;---------------------------------------
    sta     HMOVE           ; 3 =  3    @02     one cycle to early, but that doesn't matter

; draw car #0:
    lda     #CARHEIGHT-1    ; 2
    dcp     P0_Y            ; 5
    bcs     .doDraw0        ; 2
    lda     #0              ; 2
    .byte   $2c             ;-1
.doDraw0:
    lda     (P0_Ptr),y      ; 5
    sta     GRP0            ; 3 = 18    @20

; draw playfield:
    lda     PF1Data,x       ; 4
    sta     PF1             ; 3
    lda     PF2Data,x       ; 4
    sta     PF2             ; 3
    lda     PF1bData,x      ; 4
    sta     PF1             ; 3
    lda     PF2bData,x      ; 4
    sta     PF2             ; 3 = 28    @48

; clear all hmov registers:
    sta     HMCLR           ; 3 =  3    @51

;    SLEEP   -1

; draw zombie #1:
    lda     #ZOMBIE_H-1     ; 2
    dcp     M1_Y            ; 5
    beq     .enableM1       ; 2
    bcc     .disableM1      ; 2
    lda     (M1_Ptr),y      ; 5                 bytes stored as: mmmmss00
    sta     HMM1            ; 3
    asl                     ; 2
    asl                     ; 2
    sta     NUSIZ1          ; 3         @01
.continueM1:                ;   = 26            timing is constant!

;    sta     WSYNC
;---------------------------------------
    sta     HMOVE           ; 3 =  3    @04     one cycle to late, but that doesn't matter

; draw car #1:
    lda     #CARHEIGHT-1    ; 2
    dcp     P1_Y            ; 5
    bcs     .doDraw1        ; 2
    lda     #0              ; 2
    .byte   $2c             ;-1
.doDraw1:
    lda     (P1_Ptr),y      ; 5
    sta     GRP1            ; 3 = 18    @22

; disable playfield:
    lda     #0              ; 2
    sta     PF1             ; 3         @27
    sta     PF2             ; 3 =  8    @30

; clear all hmov registers:
    sta     HMCLR           ; 3 =  3    @33

; TombstoneIndexer routine:
    dey                     ; 2
    tya                     ; 2
    and     #7              ; 2
    bne     .contBlock      ; 2
    dex                     ; 2
    bpl     .nextBlock      ; 2

    jmp     ExitKernel
;***************************************

.enableM1:                  ;10
    lda     (M1_Ptr),y      ; 5                 bytes stored as: mmmmss00
    sta     HMM1            ; 3
    lda     #ENABLE_BM      ; 2
    sta     ENAM1           ; 3
    bne     .continueM1     ; 3

.disableM1:                 ;12
    nop                     ; 2                 waste six cycles for constant timing
    nop                     ; 2
    nop                     ; 2
    lda     #DISABLE_BM     ; 2
    sta     ENAM1           ; 3
    beq     .continueM1     ; 3
;***************************************

    align   256

Kernel10 SUBROUTINE

.enableM1:                  ;10
    lda     (M1_Ptr),y      ; 5                 bytes stored as: mmmmss00
    sta     HMM1            ; 3
    lda     #ENABLE_BM      ; 2
    sta     ENAM1           ; 3
    bne     .continueM1     ; 3

.disableM1:                 ;12
    nop                     ; 2                 waste six cycles for constant timing
    nop                     ; 2
    nop                     ; 2
    lda     #DISABLE_BM     ; 2
    sta     ENAM1           ; 3
    beq     .continueM1     ; 3
;***************************************

EnterKernel10:              ;           @42
.contBlock:
    nop                     ; 2
    nop                     ; 2
.nextBlock:                 ;   = 13

    SLEEP   3               ; 3 =  3

; draw zombie #1:
    lda     #ZOMBIE_H-1     ; 2
    dcp     M1_Y            ; 5
    beq     .enableM1       ; 2
    bcc     .disableM1      ; 2
    lda     (M1_Ptr),y      ; 5                 bytes stored as: mmmmss00
    sta     HMM1            ; 3
    asl                     ; 2
    asl                     ; 2
    sta     NUSIZ1          ; 3         @75
.continueM1:                ;   = 26            timing is constant!

;    sta     WSYNC
;---------------------------------------
    sta     HMOVE           ; 3 =  3    @02     one cycle to early, but that doesn't matter

; draw car #0:
    lda     #CARHEIGHT-1    ; 2
    dcp     P0_Y            ; 5
    bcs     .doDraw0        ; 2
    lda     #0              ; 2
    .byte   $2c             ;-1
.doDraw0:
    lda     (P0_Ptr),y      ; 5
    sta     GRP0            ; 3 = 18    @20

; draw playfield:
    lda     PF1Data,x       ; 4
    sta     PF1             ; 3
    lda     PF2Data,x       ; 4
    sta     PF2             ; 3
    lda     PF1bData,x      ; 4
    sta     PF1             ; 3
    lda     PF2bData,x      ; 4
    sta     PF2             ; 3 = 28    @48

; clear all hmov registers:
    sta     HMCLR           ; 3 =  3    @51

;    SLEEP   -1

; draw zombie #0:
    lda     #ZOMBIE_H-1     ; 2
    dcp     M0_Y            ; 5
    beq     .enableM0       ; 2
    bcc     .disableM0      ; 2
    lda     (M0_Ptr),y      ; 5                 bytes stored as: mmmmss00
    sta     HMM0            ; 3
    asl                     ; 2
    asl                     ; 2
    sta     NUSIZ0          ; 3         @01
.continueM0:                ;   = 26            timing is constant!

;    sta     WSYNC
;---------------------------------------
    sta     HMOVE           ; 3 =  3    @04     one cycle to late, but that doesn't matter

; draw car #1:
    lda     #CARHEIGHT-1    ; 2
    dcp     P1_Y            ; 5
    bcs     .doDraw1        ; 2
    lda     #0              ; 2
    .byte   $2c             ;-1
.doDraw1:
    lda     (P1_Ptr),y      ; 5
    sta     GRP1            ; 3 = 18    @22

; disable playfield:
    lda     #0              ; 2
    sta     PF1             ; 3         @27
    sta     PF2             ; 3 =  8    @30

; clear all hmov registers:
    sta     HMCLR           ; 3 =  3    @33

; TombstoneIndexer routine:
    dey                     ; 2
    tya                     ; 2
    and     #7              ; 2
    bne     .contBlock      ; 2
    dex                     ; 2
    bpl     .nextBlock      ; 2

    jmp     ExitKernel
;***************************************

.enableM0:                  ;10
    lda     (M0_Ptr),y      ; 5                 bytes stored as: mmmmss00
    sta     HMM0            ; 3
    lda     #ENABLE_BM      ; 2
    sta     ENAM0           ; 3
    bne     .continueM0     ; 3

.disableM0:                 ;12
    nop                     ; 2                 waste six cycles for constant timing
    nop                     ; 2
    nop                     ; 2
    lda     #DISABLE_BM     ; 2
    sta     ENAM0           ; 3
    beq     .continueM0     ; 3
;***************************************

ExitKernel:
    sta     WSYNC
;---------------------------------------
    sta     PF0             ;                   a = 0
    sta     PF1
    sta     PF2
    sta     GRP0
    sta     GRP1
    sta     GRP0
    sta     ENAM0
    sta     ENAM1
    sta     NUSIZ0
    sta     NUSIZ1

    ldx     #2
.waitScreen0:
    lda     INTIM
    bne     .waitScreen0
    sta     WSYNC
    stx     VBLANK
    rts


OverScan SUBROUTINE
    lda     #36
    sta     TIM64T

;   ...

.waitTim:
    lda     INTIM
    bne     .waitTim
    rts


SetPosX SUBROUTINE
; TODO: use new Fatal Run method discovered by Manuel
; calculates the values and positions objects:
; Input:
; - a = x-position
    tay                      ; 2
    iny                      ; 2
    tya                      ; 2
    and    #$0F              ; 2
    sta    tmpVar            ; 3
    tya                      ; 2
    lsr                      ; 2
    lsr                      ; 2
    lsr                      ; 2
    lsr                      ; 2
    tay                      ; 2
    clc                      ; 2
    adc    tmpVar            ; 3
    cmp    #$0F              ; 2
    bcc    .skipIny          ; 2
    sbc    #$0F              ; 2
    iny                      ; 2
.skipIny:
    sta    WSYNC             ; 3
    eor    #$07              ; 2
    asl                      ; 2
    asl                      ; 2
    asl                      ; 2
    asl                      ; 2
    sta.wx  HMP0,x           ; 5        @15
.waitPos:
    dey                      ; 2
    bpl    .waitPos          ; 2
.bankCheck:
    sta    RESP0,X           ; 4
    rts                      ; 6
;SetPosX END


;===============================================================================
; R O M - T A B L E S (Bank 0)
;===============================================================================
    org     $ff00

PF1Data  ;D    76543210
    .byte %01111001
    .byte %01000101
    .byte %01000101
    .byte %01000101
    .byte %01111001
    .byte %00000000
    .byte %01111001
    .byte %01000101
    .byte %01000101
    .byte %01000101
    .byte %01111001

PF2Data  ;EA   01234567
    .byte %00101111
    .byte %10100000
    .byte %11100111
    .byte %00100000
    .byte %11101111
    .byte %00000000
    .byte %00101111
    .byte %00100000
    .byte %11100111
    .byte %00100000
    .byte %11101111

PF2bData ;     76543210;<--- scanning order
    .byte %11011110
    .byte %00010001
    .byte %10011110
    .byte %01010001
    .byte %10011110
    .byte %00000000
    .byte %01000100
    .byte %01000100
    .byte %11000100
    .byte %01000100
    .byte %11011111

PF1bData;      01234567;<--- scanning order
    .byte %00111110
    .byte %00100000
    .byte %00111110
    .byte %00100010
    .byte %00100010
    .byte %00000000
    .byte %00100010
    .byte %00100010
    .byte %00111110
    .byte %00100010
    .byte %00100010

KernelLoTbl:
    .byte   <EnterKernel00
    .byte   <EnterKernel10
    .byte   <EnterKernel00   ; preliminary!
    .byte   <EnterKernel10

KernelHiTbl:
    .byte   >EnterKernel00
    .byte   >EnterKernel10
    .byte   >EnterKernel00   ; preliminary!
    .byte   >EnterKernel10



    org     $ff00+TOMBSTONEROWS*TOMBSTONEHEIGHT

CarRotation0;UP (ACTUAL)
  ;.byte %00000000
  .byte %01011010; .X.XX.X.
  .byte %01111110; .XXXXXX.
  .byte %01100110; .XX..XX.
  .byte %00011000; ...XX...
  .byte %01011010; .X.XX.X.
  .byte %01111110; .XXXXXX.
  .byte %01011010; .X.XX.X.
  .byte %00011000; ...XX...

CarRotation1
  ;.byte %00000000
  .byte %00000110; ....XX..
  .byte %00011110; ..XXXX..
  .byte %01110100; XXX.X...
  .byte %00011010; XX.XX.X.
  .byte %00011010; ...XX.X.
  .byte %01011111; .X.XXXXX
  .byte %00111100; ..XXXX..
  .byte %00101100; ..X.XX..

CarRotation2
  ;.byte %00000000
  .byte %00011000; ...XX...
  .byte %01111000; .XXXX...
  .byte %01000010; .X....X.
  .byte %11011011; XX.XX.XX
  .byte %11011100; XX.XXX..
  .byte %00011110; ...XXXX.
  .byte %00110110; ..XX.XX.
  .byte %00010000; ...X....

CarRotation3
  ;.byte %00000000
  .byte %00110000; ..XX....
  .byte %00110010; ..XX.X..
  .byte %01100011; .XX...XX
  .byte %01011110; .X.XXXX.
  .byte %11111111; XXXXXXXX
  .byte %11000111; XX...XXX
  .byte %00011100; ...XXX..
  .byte %00000100; .....X..

CarRotation4;RIGHT|LEFT
  ;.byte %00000000
  .byte %00000000; ........
  .byte %11101110; XXX.XXX.
  .byte %01100100; .XX..X..
  .byte %11011111; XX.XXXXX
  .byte %11011111; XX.XXXXX
  .byte %01100100; .XX..X..
  .byte %11101110; XXX.XXX.
  .byte %00000000; ........

CarRotation5
  ;.byte %00000000
  .byte %00000100; .....X..
  .byte %00011100; ...XXX..
  .byte %11000111; XX...XXX
  .byte %11111111; XXXXXXXX
  .byte %01011110; .X.XXXX.
  .byte %01100011; .XX...XX
  .byte %00110010; ..XX.X..
  .byte %00110000; ..XX....

CarRotation6
  ;.byte %00000000
  .byte %00010000; ...X....
  .byte %00110110; ..XX.XX.
  .byte %00011110; ...XXXX.
  .byte %11011100; XX.XXX..
  .byte %11011011; XX.XX.XX
  .byte %01000010; .X....X.
  .byte %01111000; .XXXX...
  .byte %00011000; ...XX...

CarRotation7
  ;.byte %00000000
  .byte %00101100; ..X.XX..
  .byte %00111100; ..XXXX..
  .byte %01011111; .X.XXXXX
  .byte %00011010; ...XX.X.
  .byte %11011010; XX.XX.X.
  .byte %11101000; XXX.X...
  .byte %00111100; ..XXXX..
  .byte %00001100; ....XX..

CarRotation8;DOWN (ACTUAL)
  ;.byte %00000000
  .byte %00011000; ...XX...
  .byte %01011010; .X.XX.X.
  .byte %01111110; .XXXXXX.
  .byte %01011010; .X.XX.X.
  .byte %00011000; ...XX...
  .byte %01100110; .XX..XX.
  .byte %01111110; .XXXXXX.
  .byte %01011010; .X.XX.X.


Ped1_1;   OOOOWW__
   .byte HMOVE_L1|BM_SIZE_2/4 ; .XX.....
   .byte HMOVE_L1|BM_SIZE_2/4 ; ..XX....
   .byte HMOVE_0 |BM_SIZE_4/4 ; ...XXXX.
   .byte HMOVE_R2|BM_SIZE_2/4 ; ...XX...
   .byte HMOVE_L3|BM_SIZE_4/4 ; .XXXX...
   .byte HMOVE_R1|BM_SIZE_1/4 ; ....X...
   .byte HMOVE_0 |BM_SIZE_2/4 ; ...XX...
   .byte HMOVE_0 |BM_SIZE_2/4 ; ...XX...

Ped1_2;   OOOOWW__
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...
   .byte %00000000 ; ........
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...

Ped1_3;   OOOOWW__
   .byte %11110100 ; .XX.....
   .byte %00000000 ; ..X.....
   .byte %11011100 ; ...XXXX.
   .byte %11100100 ; ..XX....
   .byte %11010100 ; ...XX...
   .byte %00000000 ; ........
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...

Ped1_4;   OOOOWW__
   .byte HMOVE_0 |BM_SIZE_2/4 ; ...XXXX.
   .byte HMOVE_0 |BM_SIZE_1/4 ; ...X....
   .byte HMOVE_0 |BM_SIZE_4/4 ; ...XXXX.
   .byte HMOVE_R3|BM_SIZE_2/4 ; ...XX...
   .byte HMOVE_L2|BM_SIZE_4/4 ; XXXX....
   .byte HMOVE_R1|BM_SIZE_4/4 ; ..XXXX..
   .byte HMOVE_0 |BM_SIZE_2/4 ; .XX.....
   .byte HMOVE_0 |BM_SIZE_2/4 ; .XX.....

    org $fffc
    .word   Start
    .word   Start
